import requests
import json
import twocheckout

class Request(object):

    @classmethod
    def call(cls, method, path, args=None):
        url = cls.build_url(path)
        headers = cls.build_headers()

        if method == 'get':
            params = args
            data = None
        else:
            params = None
            data = json.dumps(args) if args else None

        r = requests.request(method, url, headers=headers, params=params, data=data, auth=(twocheckout.seller_id, twocheckout.private_key))
        response = json.loads(r.text)
        cls.check_for_error(r.status_code, response)
        return response

    @classmethod
    def build_headers(cls):
        headers = {
            'Accept': 'application/json',
            'User-Agent': '2Checkout Python/0.1.0/%s',
            'Content-Type': 'application/json'
        }
        return headers

    @classmethod
    def build_url(cls, path):
        if twocheckout.sandbox:
            url = twocheckout.sandbox_base_url
        else:
            url = twocheckout.base_url
        url += path
        return str(url)

    @classmethod
    def check_for_error(cls, code, response):
        if code != 200:
            raise twocheckout.TwoCheckoutError(
                response['error']['code'],
                response['error']['message'],
                response['error']['param']
            )
